#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e6|e7|e9|e10>:<zlib|nofcpp> [debug|prof] [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

die()
{
    local exitval
    if [[ "$1" =~ ^[0-9]*$ ]]; then (( exitval = $1 )); shift; else (( exitval = 1 )); fi
    echo "ERROR: $@" 1>&2
    exit $exitval
}

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
        case $qual in
            e[1-9]*)
                cqual=$qual
                ;;
            nofcpp)
                eval $qual=\$qual
                confoptions=(--disable-cxx --disable-fortran --disable-dependency-tracking)
                ;;
            zlib)
                eval $qual=\$qual
                confoptions=(--enable-cxx --enable-fortran --with-zlib=\${ZLIB_INC},\${ZLIB_LIB})
                ;;
            *)
                die "Unrecognized qualifier $qual"
        esac
    done
    if [[ -z "$cqual" ]]; then
        die "Base qualifier is mandatory"
    fi
    if [[ -n "$zlib" ]] && [[ -n "$nofcpp" ]]; then
      die "Qualifiers $zlib and $nofcpp are mutually exclusive"
    elif [[ -z "$zlib" ]] && [[ -z "$nofcpp" ]]; then
      die "zlib or nofcpp qualifier is required"
    fi
    if [[ -z "$extraqual" ]] && [[ -n "$nofcpp" ]]; then
      die "debug|prof is mandatory with $nofcpp"
    elif [[ -n "$zlib" ]] && [[ -n "$extraqual" ]]; then
      die "$extraqual not compatible with $zlib"
    fi
    fullqual=${cqual}:${zlib}${nofcpp}${extraqual:+:${extraqual}}
    qualdir=`echo ${fullqual} | sed -e 's/:/-/g'`
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

set_compiler_flags()
{
    case ${cqual} in
        e6)
            cxxflags="-std=c++1y"
            ;;
        e[79]|e10)
            cxxflags="-std=c++14"
            ;;
        *)
            die "Unrecognized compiler qualifier $cqual"
    esac
    [[ -n "${extraqual}" ]] && gflag="-g"
    case x$extraqual in
        xdebug)
            prodflag=--disable-production
            ;;
        xprof)
            prodflag=--enable-production            
            ;;
        x)
            ;;
        *)
            die "Unrecognized extra qualifier $extraqual"            
    esac
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
if [[ "$extraqual" == "tar" ]]; then
  unset extraqual
  maketar=tar
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=hdf5
origpkgver=v1_8_16
pkgver=${origpkgver}
ssibuildshims_version=v0_09_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims
set_qualifiers
set_compiler_flags
get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "fake setup failed"

set -x

cd "${sourcedir}" && rm -rf hdf5-${pkgdotver} || die "Unable to cd into ${sourcedir}"
tar xzf ${tardir}/${pkgtarname}* || die "Unable to untar source"

cd "${blddir}" || die "Unable to cd into ${blddir}"

eval ${cxxflags:+CXXFLAGS=${cxxflags}} ${gflag:+CFLAGS=${gflag}} \
${sourcedir}/hdf5-${pkgdotver}/configure --prefix="${pkgdir}" "${confoptions[@]}" ${prodflag} || \
  die "configure failed"

make -j || die "make failed"

make -j check || die "make check failed"

make install || die "make install failed"

make -j check-install || die "make check-install failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "hdf5 is installed at ${HDF5_FQ_DIR}"


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
